Type VayPoint
	Field x#, y#, z#
	Field radius#
	Field state%
	Field cntVP, nextVP[10]
	Field mesh
End Type

Type EntVP
	Field ent
	Field spTurn#, spMove#
	Field curVP
	Field state
	Field timeLeft
End Type

Const ST_GO=1, ST_WAIT=2

Function AddVayPoint(x#, y#, z#, radius#=1.0, state=ST_GO)
	Local vp.VayPoint = New VayPoint
	
	TerrY#=TerrainY(Terrain,x#,y#,z#)+.7
	VP\x = x
	VP\y = TerrY
	VP\z = z
	
	VP\radius# = radius
	VP\state = state
	If debug = True Then
		VP\mesh = CopyEntity(debugMesh)
		PositionEntity VP\mesh, x, TerrY#, z
	EndIf
	
	Return Handle(VP)
End Function
Function AddNextVP(handleVP1, handleVP2)
	Local VP1.VayPoint = Object.VayPoint(handleVP1)
	
	VP1\nextVP[VP1\cntVP] = handleVP2
	VP1\cntVP = VP1\cntVP + 1
End Function
Function AddEntityToVP(ent, spTurn#=0.9, spMove# = 0.1)
	Local e.EntVP = New EntVP
	e\ent = ent
	e\spTurn = spTurn
	e\spMove = spMove
	
	Return Handle(e)
End Function

Function SetEntityVP(entVP, handleVP)
	Local e.EntVP = Object.EntVP(entVP)
	e\curVP = handleVP
	Local VP.VayPoint = Object.VayPoint(e\curVP)
	e\state = VP\state
End Function
Function GetEntityState(entVP)
	Local e.EntVP = Object.EntVP(entVP)
	;If e = Null Then RuntimeError( "VP entity not found" )
	Return e\state
End Function
Function UpdateVP()
	Local e.EntVP, VP.VayPoint
	Local dx#, dy#, dz#, pit#, yaw#
	For e.EntVP = Each EntVP
		If e\state = ST_GO Then
			VP.VayPoint = Object.VayPoint(e\curVP)
			If VP <> Null Then
				dx# = VP\x - EntityX(e\ent,1)
				dy# = VP\y - EntityY(e\ent,1)
				dz# = VP\z - EntityZ(e\ent,1)
				If Abs(dx)<VP\radius And Abs(dy)<VP\radius And Abs(dz)<VP\radius Then
					Select VP\state
						Case ST_GO
							e\curVP = VP\nextVP[Rand(0,VP\cntVP-1)]
							e\state = VP\state
							If e\curVP <= 0 Then e\state = 0
						Case ST_WAIT
							e\timeLeft = MilliSecs() + Rand(3,15)*1000
							e\state = VP\state
							e\curVP = VP\nextVP[Rand(0,VP\cntVP-1)]
							If e\curVP <= 0 Then e\state = 0
					End Select
				Else
					pit# = VectorPitch(dx, dy, dz)
					yaw# = VectorYaw(dx, dy, dz)
					pit = MergeAngles#(EntityPitch(e\ent,1),pit,e\spTurn)
					yaw = MergeAngles#(EntityYaw(e\ent,1),yaw,e\spTurn)
					RotateEntity e\ent, pit, yaw, EntityRoll(e\ent,1)
					MoveEntity e\ent, 0, 0, e\spMove
					CheckTerrainY(Terrain,e\ent,VP\x,VP\y,VP\z)
				EndIf
			Else
				;RuntimeError( "Vaypoint not found" )
			EndIf
		ElseIf e\state = ST_WAIT Then
			If MilliSecs() > e\timeLeft Then
				e\state = ST_GO
			EndIf
		EndIf
	Next
End Function
Function MergeAngles#(Angle1#,Angle2#,Morph#=.5)
	If Abs(Angle1-Angle2)>180
		If Angle1<Angle2
			Return Angle1-((360-Angle2)+Angle1)*(.5-(Morph-.5))
		Else
			Return Angle2-((360-Angle1)+Angle2)*(.5+(Morph-.5))
		EndIf
	Else
		Return Angle1*(Morph)+Angle2*(1.0-Morph)
	EndIf
End Function